# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xoinvader', 'xoinvader.tests']

package_data = \
{'': ['*'],
 'xoinvader': ['config/*', 'res/*', 'res/gfx/*', 'res/tgfx/*'],
 'xoinvader.tests': ['fixtures/*']}

install_requires = \
['eaf>=0.2,<0.3', 'toml>=0.10,<0.11', 'xo1>=0.1,<0.2']

entry_points = \
{'console_scripts': ['xoigame = xoinvader.game:main']}

setup_kwargs = {
    'name': 'xoinvader',
    'version': '0.2.0',
    'description': '(not so) small python curses space game',
    'long_description': '|PyPI| |Build Status| |codecov.io|\n\n=========\nXOInvader\n=========\n\nAttempt to create small but serious game using **python** and\n**ncurses**.\n\nRequirements\n============\n\nGeneral\n-------\n\n* >= python-3.7\n* >= eaf-0.2\n* >= xo1-0.1\n* >= toml-0.10\n* ncurses >=5.9\n\nInstallation\n============\n\nFrom PyPI\n---------\n\n.. code-block:: console\n\n   $ pip install --user xoinvader\n\n\nDevelopment\n-----------\n\n.. code-block:: console\n\n   $ poetry install\n   $ poetry run xoigame\n\nTesting\n-------\n\n.. code-block:: console\n\n   $ make test  # for all tests\n   $ make view_cov  # run all tests and view coverage\n   $ # pass additional arguments to pytest via PYTEST_ARGS env variable\n   $ # you can pass args such way to test and view_cov make targets\n   $ make test PYTEST_ARGS=\'-m "not slow"\'  # don\'t run slow tests\n   $ pytest --markers  # show all markers\n   $ pytest xoinvader/path/to/test_file.py  # run separate file\n\n\nDocumentation\n-------------\n\n* Autogenerated from sources `here <http://xoinvader.mmap.me/>`__.\n* Development `wiki <https://github.com/pkulev/xoinvader/wiki/>`_.\n* Build documentation from sources:\n\n  .. code-block:: console\n\n     $ make docs\n     $ make view_docs  # build and open in firefox\n\n\n.. |PyPI| image:: https://badge.fury.io/py/xoinvader.svg\n   :target: https://badge.fury.io/py/xoinvader\n.. |Build Status| image:: https://travis-ci.org/pkulev/xoinvader.svg?branch=master\n   :target: https://travis-ci.org/pkulev/xoinvader\n.. |codecov.io| image:: http://codecov.io/github/pkulev/xoinvader/coverage.svg?branch=master\n   :target: http://codecov.io/github/pkulev/xoinvader?branch=master\n',
    'author': 'Pavel Kulyov',
    'author_email': 'kulyov.pavel@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pkulev/xoinvader',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
